package com.fr.plugin.decision.integration.config;

import com.fr.cert.token.Jwts;
import com.fr.cert.token.SignatureAlgorithm;
import com.fr.config.ConfigContext;
import com.fr.config.DefaultConfiguration;
import com.fr.config.Identifier;
import com.fr.config.holder.Conf;
import com.fr.config.holder.factory.Holders;
import com.fr.config.utils.ValueReader;
import com.fr.config.utils.ValueWriter;
import com.fr.decision.base.util.UUIDUtil;
import com.fr.json.JSONObject;
import com.fr.plugin.decision.integration.config.strategy.CreateDepartmentStrategy;
import com.fr.plugin.decision.integration.config.strategy.CreatePwdStrategy;
import com.fr.plugin.decision.integration.config.strategy.user.impl.ContainDepRoleStrategy;
import com.fr.plugin.decision.integration.config.strategy.user.impl.OnlyUserStrategy;
import com.fr.plugin.decision.integration.config.strategy.user.CreateUserStrategy;
import com.fr.plugin.decision.integration.utils.AESUtils;
import com.fr.plugin.decision.integration.utils.Constants;
import com.fr.stable.StringUtils;
import com.fr.third.fasterxml.jackson.annotation.JsonIgnoreProperties;

import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.Base64;
import java.util.Date;
import java.util.HashMap;

@JsonIgnoreProperties({"id", "data", "classInfo", "nameSpace"})
public class IntegrateConf extends DefaultConfiguration {
    private static volatile IntegrateConf configuration = null;

    public static IntegrateConf getInstance() {
        if (null == configuration) {
            configuration = ConfigContext.getConfigInstance(IntegrateConf.class);
        }
        return configuration;
    }

    @Identifier("jwtKey")
    private Conf<String> jwtKey = Holders.simple(UUIDUtil.generate());

    @Identifier("createUserTurnOn")
    private Conf<Boolean> createUserTurnOn = Holders.simple(false);

    @Identifier("userStrategy")
    private Conf<CreateUserStrategy> userStrategy = Holders.obj(new OnlyUserStrategy(), CreateUserStrategy.class);

    @Identifier("paraCheckTurnOn")
    private Conf<Boolean> paraCheckTurnOn = Holders.simple(false);

    @Identifier("paraCheckUrl")
    private Conf<String> paraCheckUrl = Holders.simple(StringUtils.EMPTY);

    @Identifier("remoteTokenAuth")
    private Conf<Boolean> remoteTokenAuth = Holders.simple(true);

    @Identifier("aesEncryptSubject")
    private Conf<Boolean> aesEncryptSubject = Holders.simple(false);

    @Identifier("aesEncryptKey")
    private Conf<String> aesEncryptKey = Holders.simple(StringUtils.EMPTY);

    @Identifier("portalURLTurnOn")
    private Conf<Boolean> portalURLTurnOn = Holders.simple(false);

    @Identifier("portalURL")
    private Conf<String> portalURL = Holders.simple(StringUtils.EMPTY);

    @Identifier("crossDomain")
    private Conf<Boolean> crossDomain = Holders.simple(false);

    @Identifier("webStartDesigner")
    private Conf<Boolean> webStartDesigner = Holders.simple(false);

    @Identifier("downloadUrl32")
    private Conf<String> downloadUrl32 = Holders.simple("");

    @Identifier("downloadUrl64")
    private Conf<String> downloadUrl64 = Holders.simple("");

    @Identifier("decisionUrl")
    private Conf<String> decisionUrl = Holders.simple(StringUtils.EMPTY);

    @Identifier("remoteFolder")
    private Conf<Boolean> remoteFolder = Holders.simple(false);

    @Identifier("ignoreMobileCookie")
    private Conf<Boolean> ignoreMobileCookie = Holders.simple(false);

    public String getJwtKey() {
        return jwtKey.get();
    }

    public void setJwtKey(String jwtKey) {
        this.jwtKey.set(jwtKey);
    }

    public Boolean getCreateUserTurnOn() {
        return createUserTurnOn.get();
    }

    public void setCreateUserTurnOn(Boolean createUserTurnOn) {
        this.createUserTurnOn.set(createUserTurnOn);
    }

    public CreateUserStrategy getUserStrategy() {
        return userStrategy.get();
    }

    public void setUserStrategy(CreateUserStrategy userStrategy) {
        this.userStrategy.set(userStrategy);
    }

    public Boolean getParaCheckTurnOn() {
        return paraCheckTurnOn.get();
    }

    public void setParaCheckTurnOn(Boolean paraCheckTurnOn) {
        this.paraCheckTurnOn.set(paraCheckTurnOn);
    }

    public String getParaCheckUrl() {
        return paraCheckUrl.get();
    }

    public void setParaCheckUrl(String paraCheckUrl) {
        this.paraCheckUrl.set(paraCheckUrl);
    }

    public Boolean getRemoteTokenAuth() {
        return remoteTokenAuth.get();
    }

    public void setRemoteTokenAuth(Boolean remoteTokenAuth) {
        this.remoteTokenAuth.set(remoteTokenAuth);
    }

    public Boolean getAesEncryptSubject() {
        return aesEncryptSubject.get();
    }

    public void setAesEncryptSubject(Boolean aesEncryptSubject) {
        this.aesEncryptSubject.set(aesEncryptSubject);
    }

    public String getAesEncryptKey() {
        return aesEncryptKey.get();
    }

    public void setAesEncryptKey(String aesEncryptKey) {
        this.aesEncryptKey.set(aesEncryptKey);
    }

    public Boolean getPortalURLTurnOn() {
        return portalURLTurnOn.get();
    }

    public void setPortalURLTurnOn(Boolean portalURLTurnOn) {
        this.portalURLTurnOn.set(portalURLTurnOn);
    }

    public String getPortalURL() {
        return portalURL.get();
    }

    public void setPortalURL(String portalURL) {
        this.portalURL.set(portalURL);
    }

    public Boolean getCrossDomain() {
        return crossDomain.get();
    }

    public void setCrossDomain(Boolean crossDomain) {
        this.crossDomain.set(crossDomain);
    }

    public Boolean getWebStartDesigner() {
        return webStartDesigner.get();
    }

    public void setWebStartDesigner(Boolean webStartDesigner) {
        this.webStartDesigner.set(webStartDesigner);
    }

    public String getDownloadUrl32() {
        return downloadUrl32.get();
    }

    public void setDownloadUrl32(String downloadUrl32) {
        this.downloadUrl32.set(downloadUrl32);
    }

    public String getDownloadUrl64() {
        return downloadUrl64.get();
    }

    public void setDownloadUrl64(String downloadUrl64) {
        this.downloadUrl64.set(downloadUrl64);
    }

    public String getDecisionUrl() {
        return decisionUrl.get();
    }

    public void setDecisionUrl(String decisionUrl) {
        this.decisionUrl.set(decisionUrl);
    }

    public Boolean getRemoteFolder() {
        return remoteFolder.get();
    }

    public void setRemoteFolder(Boolean remoteFolder) {
        this.remoteFolder.set(remoteFolder);
    }

    public Boolean getIgnoreMobileCookie() {
        return ignoreMobileCookie.get();
    }

    public IntegrateConf setIgnoreMobileCookie(Boolean ignoreMobileCookie) {
        this.ignoreMobileCookie.set(ignoreMobileCookie);
        return this;
    }

    @Override
    protected void initialize() {
        ValueWriter.registerWriter(CreateDepartmentStrategy.class, new ValueWriter<CreateDepartmentStrategy>() {
            @Override
            public String writeObject(CreateDepartmentStrategy createDepEnum) {
                return String.valueOf(createDepEnum.toInteger());
            }
        });
        ValueReader.registerReader(CreateDepartmentStrategy.class, new ValueReader<CreateDepartmentStrategy>() {
            public CreateDepartmentStrategy covert(String value) {
                return CreateDepartmentStrategy.parse(Integer.parseInt(value));
            }
        });
        ValueWriter.registerWriter(CreatePwdStrategy.class, new ValueWriter<CreatePwdStrategy>() {
            @Override
            public String writeObject(CreatePwdStrategy createDepEnum) {
                return String.valueOf(createDepEnum.toInteger());
            }
        });
        ValueReader.registerReader(CreatePwdStrategy.class, new ValueReader<CreatePwdStrategy>() {
            public CreatePwdStrategy covert(String value) {
                return CreatePwdStrategy.parse(Integer.parseInt(value));
            }
        });
    }

    public HashMap getWebStartedInfo(String username) throws Exception {
        HashMap<String, String> info = new HashMap<>(3);
        info.put("url", "fanruan://" + getDecisionUrl() + "?GWToken?" + getTokenByUserName(username));
        info.put("download64", getDownloadUrl64());
        info.put("download32", getDownloadUrl32());
        return info;
    }

    public String getWebStartURL(String username, String path) throws Exception {
        StringBuffer buffer = new StringBuffer();
        buffer.append("<script>window.open('")
                .append("fanruan://").append(getDecisionUrl()).append("?GWToken?").append(getTokenByUserName(username))
                .append(StringUtils.isEmpty(path)? "": ("?" + URLEncoder.encode(path, StandardCharsets.UTF_8.name())))
                .append("');(window.top==window)&&window.close();</script>");
        return buffer.toString();
    }

    private String getTokenByUserName(String username) throws Exception {
        String subject = username;
        if (getUserStrategy() instanceof ContainDepRoleStrategy) {
            subject = JSONObject.create().put("username", username).toString();
        }
        if (getAesEncryptSubject()) {
            subject = Base64.getEncoder().encodeToString(AESUtils.aesEncryptToBytes(subject, getAesEncryptKey()));
        }
        Date date = new Date();
        return Jwts.builder()
                .setSubject(subject)
                .setExpiration(new Date(date.getTime() + Constants.EXPIRED_TIME))
                .signWith(SignatureAlgorithm.HS256, getJwtKey())
                .compact();
    }
}
