package com.fr.plugin.decision.integration.filter;

import com.fr.data.NetworkHelper;
import com.fr.decision.fun.impl.AbstractEmbedRequestFilterProvider;
import com.fr.decision.webservice.Response;
import com.fr.decision.webservice.v10.remote.RemoteDesignService;
import com.fr.invoke.Reflect;
import com.fr.plugin.decision.integration.config.IntegrateConf;
import com.fr.plugin.decision.integration.service.IntegrateAuthService;
import com.fr.plugin.decision.integration.utils.LogUtils;
import com.fr.security.JwtUtils;
import com.fr.security.SecurityToolbox;
import com.fr.security.encryption.transmission.TransmissionEncryptors;
import com.fr.stable.StringUtils;
import com.fr.store.StateHubService;
import com.fr.third.fasterxml.jackson.databind.ObjectMapper;
import com.fr.web.utils.WebUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * 拦截设计器远程请求, 支持Token认证
 */
public class RemoteFilter extends AbstractEmbedRequestFilterProvider {

    private static final String PATH_TOKEN = "/remote/design/token";
    private static final String PATH_VERIFY = "/remote/design/verify";

    @Override
    public void filter(HttpServletRequest request, HttpServletResponse response) {
        if (!IntegrateConf.getInstance().getRemoteTokenAuth()) return;
        String username = NetworkHelper.getHTTPRequestParameter(request, "username");
        String password;
        if (request.getRequestURI().endsWith(PATH_TOKEN)) {
            if (StringUtils.equalsIgnoreCase("get", request.getMethod())) {
                password = NetworkHelper.getHTTPRequestParameter(request, "password");
            } else {
                password = SecurityToolbox.defaultDecrypt(NetworkHelper.getHTTPRequestParameter(request, "password"));
            }
        } else if (request.getRequestURI().endsWith(PATH_VERIFY)) {
            password = TransmissionEncryptors.getInstance().decrypt(NetworkHelper.getHTTPRequestParameter(request, "password"));
        } else {
            return;
        }

        String token_user = null;
        try {
            token_user = IntegrateAuthService.getInstance().getUserNameFromToken(password);
        } catch (Exception ignore) { }
        if (StringUtils.isNotBlank(username) && StringUtils.equals(token_user, username)) {
            try {
                String token = JwtUtils.createDefaultJWT(username);
                recordToken(token, username);
                WebUtils.printAsString(response, new ObjectMapper().writeValueAsString(Response.ok(token)));
            } catch (Exception e) {
                LogUtils.error(e.getMessage(), e);
            }
        }
    }

    private void recordToken(String token, String username) throws Exception {
        try {
            Reflect.on(RemoteDesignService.getInstance()).call("recordTokenStatus", username, token);
        } catch (Exception e) {
            StateHubService stateHubService = Reflect.on("com.fr.decision.webservice.v10.remote.RemoteDesignStatusService").call("loginStatusService").get();
            stateHubService.put(token, username, 1209600000);
        }
    }
}
