package com.fr.plugin.decision.integration.utils;

import com.fr.data.NetworkHelper;
import com.fr.decision.webservice.Response;
import com.fr.decision.webservice.utils.WebServiceUtils;
import com.fr.intelligence.IntelligenceException;
import com.fr.json.JSONObject;
import com.fr.log.FineLoggerFactory;
import com.fr.stable.StringUtils;
import com.fr.third.springframework.core.NestedRuntimeException;
import com.fr.web.utils.WebUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Map;

public class CommonUtils {

    public static boolean isJsonFormatStr(String str) {
        if (StringUtils.isNotBlank(str)) {
            str = str.trim();
            try {
                new JSONObject(str);
                return true;
            } catch (Exception e) {
            }
        }
        return false;
    }

    public static void showErrorPage(HttpServletResponse response, String result, String solution) {
        try {
            PrintWriter printWriter = WebUtils.createPrintWriter(response);
            Map<String, Object> map = new HashMap<>();
            map.put("result", result);
            map.put("solution", solution);
            String page = WebServiceUtils.parseWebPageResourceSafe("com/fr/web/controller/decision/entrance/resources/unavailable.html", map);
            printWriter.write(page);
            printWriter.flush();
            printWriter.close();
        } catch (Exception e) {
            LogUtils.error(e.getLocalizedMessage(), e);
        }
    }

    public static boolean isMobile(HttpServletRequest req) {
        String[] mobileArray = {"iPhone", "iPad", "android", "windows phone", "xiaomi"};
        String userAgent = req.getHeader("user-agent");
        // if (userAgent != null && userAgent.toUpperCase().contains("MOBILE")) {
        // 华为PAD的chrome浏览器userAgent中不包含mobile
        if (userAgent != null) {
            for (String mobile : mobileArray) {
                if (userAgent.toLowerCase().contains(mobile.toLowerCase())) {
                    return true;
                }
            }
        }
        //pc端企业微信
        //if (userAgent.contains("wxwork")) return true;
        FineLoggerFactory.getLogger().debug("isMobile userAgent:{} is:{}", userAgent, NetworkHelper.getDevice(req).isMobile());
        return NetworkHelper.getDevice(req).isMobile();
    }

    public static Response formatException(Exception ex) {
        String msg = ex.getMessage();
        if (StringUtils.isBlank(msg)) {
            msg = getStackTraceInfo(ex);
        }

        if (ex instanceof IntelligenceException) {
            return Response.error(((IntelligenceException) ex).errorCode(), msg);
        }

        if (ex instanceof NestedRuntimeException) {
            msg = "Request data format exception!";
        }

        return Response.error("", msg);
    }

    //从异常中获取具体的堆栈信息
    public static String getStackTraceInfo(Exception ex) {
        StackTraceElement[] stackElements = ex.getStackTrace();
        StringBuilder stringBuilder = new StringBuilder("\r\n");
        if (stackElements != null) {
            for (StackTraceElement stackElement : stackElements) {
                stringBuilder.append(stackElement.getClassName()).append(".").append(stackElement.getMethodName()).append("\r\n");
            }
            return stringBuilder.toString();
        }
        return ex.toString();
    }
}
