package com.fr.plugin.decision.integration.utils;

import com.fr.log.FineLoggerFactory;
import com.fr.log.FineLoggerProvider;
import com.fr.plugin.context.PluginContexts;
import com.fr.stable.StringUtils;

public final class LogUtils {
    private static String LOG_PREFIX = "[平台集成] ";
    private static final String PLUGIN_VERSION;
    private static final FineLoggerProvider LOGGER = FineLoggerFactory.getLogger();

    static {
        // 判断 null
        String version = PluginContexts.currentContext().getMarker().getVersion();
        if (StringUtils.isNotBlank(version)) {
            PLUGIN_VERSION = "[v" + version + "] ";
        } else {
            PLUGIN_VERSION = "[unknown version] ";
        }

        LOG_PREFIX = LOG_PREFIX + PLUGIN_VERSION;
    }

    public static void setPrefix(String prefix) {
        if (prefix != null) {
            LOG_PREFIX = prefix;
        }
    }

    public static boolean isDebugEnabled() {
        return LOGGER.isDebugEnabled();
    }

    public static void debug(String s) {
        if (isDebugEnabled()) {
            LOGGER.debug(LOG_PREFIX + s);
        }
    }

    public static void debug(String s, Object... objects) {
        if (isDebugEnabled()) {
            LOGGER.debug(LOG_PREFIX + s, objects);
        }
    }

    public static void debug(String s, Throwable throwable) {
        if (isDebugEnabled()) {
            LOGGER.debug(LOG_PREFIX + s, throwable);
        }
    }

    public static boolean isInfoEnabled() {
        return LOGGER.isInfoEnabled();
    }

    public static void info(String s) {
        LOGGER.info(LOG_PREFIX + s);
    }

    public static void info(String s, Object... objects) {
        LOGGER.info(LOG_PREFIX + s, objects);
    }

    public static void warn(String s) {
        LOGGER.warn(LOG_PREFIX + s);
    }

    public static void warn(String s, Object... objects) {
        LOGGER.warn(LOG_PREFIX + s, objects);
    }

    public static void warn(String s, Throwable throwable) {
        LOGGER.warn(LOG_PREFIX + s, throwable);
    }

    public static void warn(Throwable throwable, String s, Object... objects) {
        LOGGER.warn(throwable, LOG_PREFIX + s, objects);
    }

    public static void error(String s) {
        LOGGER.error(LOG_PREFIX + s);
    }

    public static void error(String s, Object... objects) {
        LOGGER.error(LOG_PREFIX + s, objects);
    }

    public static void error(String s, Throwable throwable) {
        LOGGER.error(LOG_PREFIX + s, throwable);
    }

    public static void error(Throwable throwable, String s, Object... objects) {
        LOGGER.error(throwable, LOG_PREFIX + s, objects);
    }
}
